#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include <time.h>

#include "ledMap.h"

#define ARRAY_SIZE(array)	(sizeof(array) / sizeof(array[0]))

#define OUT_PIN_A	((unsigned int)21)
#define OUT_PIN_B	((unsigned int)20)
#define OUT_PIN_C	((unsigned int)16)
#define OUT_PIN_D	((unsigned int)12)
#define OUT_PIN_E	((unsigned int)13)
#define OUT_PIN_F	((unsigned int)6)
#define OUT_PIN_G	((unsigned int)5)

#define IN_PIN_START	((unsigned int)26)
#define IN_PIN_STOP		((unsigned int)19)

static void aOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_A);
}
static void aOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_A);
}

static void bOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_B);
}
static void bOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_B);
}

static void cOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_C);
}
static void cOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_C);
}

static void dOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_D);
}
static void dOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_D);
}

static void eOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_E);
}
static void eOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_E);
}

static void fOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_F);
}
static void fOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_F);
}

static void gOff(volatile unsigned int *pGpio) {
	gpClr(pGpio, OUT_PIN_G);
}
static void gOn(volatile unsigned int *pGpio) {
	gpSet(pGpio, OUT_PIN_G);
}

static void ledOff(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOff, bOff, cOff, dOff, eOff, fOff, gOff
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led1(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOff, bOn, cOn, dOff, eOff, fOff, gOff
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led2(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOn, bOn, cOff, dOn, eOn, fOff, gOn
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led3(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOn, bOn, cOn, dOn, eOff, fOff, gOn
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led4(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOff, bOn, cOn, dOff, eOff, fOn, gOn
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led5(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOn, bOff, cOn, dOn, eOff, fOn, gOn
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void led6(volatile unsigned int *pGpio) {
	static void (*kLed[])(volatile unsigned int *) = {
		aOn, bOff, cOn, dOn, eOn, fOn, gOn
	};
	size_t i;
	
	for (i = 0; i < ARRAY_SIZE(kLed); i++) {
		(*kLed[i])(pGpio);
	}
}

static void ledNumber(volatile unsigned int *pGpio, unsigned int num) {
	static void (*kNumber[])(volatile unsigned int *) = {
		NULL, led1, led2, led3, led4, led5, led6
	};
	
	(*kNumber[num])(pGpio);
}

int main(void) {
	static const unsigned int kFSelOut[] = {
		OUT_PIN_A, OUT_PIN_B, OUT_PIN_C, OUT_PIN_D,
		OUT_PIN_E, OUT_PIN_F, OUT_PIN_G
	};

	volatile unsigned int *pGpio;
	
	size_t i;
	
	unsigned int num = 0;
	
	pGpio = mmapGpioStartup();
	if (pGpio == NULL) {
		return -1;
	}
	
	// LEDpGPIOo͂ɐݒ肷
	for (i = 0; i < ARRAY_SIZE(kFSelOut); i++) {
		gpFsel(pGpio, kFSelOut[i], OUT_MODE);
	}
	
	// XCb`pGPIO͂ɐݒ肷
	gpFsel(pGpio, IN_PIN_START, IN_MODE);
	gpFsel(pGpio, IN_PIN_STOP, IN_MODE);
	
	// LEDSOFFɂ
	ledOff(pGpio);
	
	// 
	srand((unsigned int)time(NULL));

	// 1łȂԂ̓[v
	while (num != 1) {
		// X^[gXCb`ONɂȂ̂҂
		while (gpLev(pGpio, IN_PIN_START) == 0) {
			usleep(100 * 1000);
		}
		
		// XgbvXCb`ONɂȂ܂ŗ\
		while (gpLev(pGpio, IN_PIN_STOP) == 0) {
			num = (rand() % 6) + 1;
			ledNumber(pGpio, num);
			usleep(100 * 1000);
		}
	}

	mmapGpioCleanup(pGpio);

	return 0;
}
